#include <cstdio>
#include <cstdlib>
#include <map>
#include <algorithm>
#include <vector>
#include <list>

using namespace std;

typedef int64_t i64;

typedef pair<int,int> pint;

int N;

#define fail (*(char*)0)

vector< vector<int> > graf;

struct st;

struct t3 { st * p; int cnt; st* leaf; };

struct st {
    int num;
    int att;
    st * parent;
    pair<int, pair<st*, st*>> broj;
    vector< t3 > ch;
    st(int x, st * par) : num(x), att(1), parent(par), ch() {}
};

vector<st*> heap;

pair<int, pair<st*,st*>> getbroj(st * tree) {
    if (tree->ch.size() == 0) {
        return make_pair(tree->att, make_pair(tree, tree));
    }
    if (tree->ch.size() == 1) {
        return make_pair(tree->ch[0].cnt + tree->att,
                         make_pair(tree->ch[0].leaf, tree));
    } else {
        return make_pair(tree->ch[0].cnt + tree->ch[1].cnt + tree->att,
                         make_pair(tree->ch[0].leaf, tree->ch[1].leaf));
    }
}

st * treeify (int x, int par = -1, st * parent = 0) {
    st * tree = new st(x, parent);

    for (int t : graf[x]) {
        if (t == par) continue;
        st * c = treeify(t, x, tree);
        if (c->ch.size() == 0)
            tree->ch.push_back({c, 1, c});
        else
            tree->ch.push_back({c, c->ch[0].cnt + 1, c->ch[0].leaf});
    }

    sort(tree->ch.begin(), tree->ch.end(), [](t3 a, t3 b) { return a.cnt > b.cnt; } );

//    printf("%d: [ ", x);
//    for (t3 c : tree->ch) {
//        printf("(%d, %d, %d) ; ", c.p ? c.p->num : -1, c.cnt, c.leaf->num);
//    }
//    printf("]\n");

    tree->broj = getbroj(tree);

    heap.push_back(tree);

    return tree;
}

void printree(st * tree) {

    for (t3 c : tree->ch) {
        printree(c.p);
    }

//    printf("%d: [ ", tree->num);
//    for (t3 c : tree->ch) {
//        printf("(%d, %d, %d) ; ", c.p ? c.p->num : -1, c.cnt, c.leaf->num);
//    }
//    printf("]\n");

}

void zeroout(st * x, st * to, st * prev = 0) {
    x->att = 0;
    if (prev != 0) {
        for (auto& c : x->ch) {
            if (c.p == prev) {
                if (prev->ch.size() == 0)
                    c = (t3){prev, 0, prev};
                else
                    c = {prev, prev->ch[0].cnt, prev->ch[0].leaf};
                break;
            }
        }
    }
    sort(x->ch.begin(), x->ch.end(), [](t3 &a, t3 &b) { return a.cnt > b.cnt; });
    x->broj = getbroj(x);
    if (x != to)
        zeroout(x->parent, to, x);
}

void reparent(st* x, st * prev) {
    if (x == 0) return;
    for (auto& c : x->ch) {
        if (c.p == prev) {
            if (prev->ch.size() == 0)
                c = (t3){prev, 0, prev};
            else
                c = {prev, prev->ch[0].cnt, prev->ch[0].leaf};
            break;
        }
    }
    sort(x->ch.begin(), x->ch.end(), [](t3 &a, t3 &b) { return a.cnt > b.cnt; });
    x->broj = getbroj(x);
    reparent(x->parent, x);
}

st * root;
vector<pair<st*,st*>> resenja;

int main() {

    scanf("%d", &N);
    graf = vector< vector<int> >(N);
    for (int i = 0; i < N - 1; i++) {
        int x,y;
        scanf("%d%d", &x, &y);
        graf[x-1].push_back(y-1);
        graf[y-1].push_back(x-1);
    }

    root = treeify(0);

    make_heap(heap.begin(), heap.end(), [](st* a, st* b){return a->broj.first < b->broj.first; });

    while (heap[0]->broj.first > 0) {
        st* b = heap[0];
//        for (st* a : heap) {
//            printf("[%d, %d]", a->num, a->broj.first);
//        }
//        printf("\nresenje: (%d -> %d)\n", b->broj.second.first, b->broj.second.second);
        resenja.push_back(b->broj.second);

        zeroout(b->broj.second.first, b);
        zeroout(b->broj.second.second, b);
        reparent(b->parent, b);

        printree(root);
//        getchar();
        make_heap(heap.begin(), heap.end(), [](st* a, st* b){return a->broj.first < b->broj.first; });
    }

    printf("%d\n", resenja.size());
    for (auto res : resenja) {
        printf("%d %d\n", res.first->num + 1, res.second->num + 1);
    }
}

/*
5
1 3
1 4
2 3
3 5

6
5 1
5 2
5 3
5 4
5 6

*/